/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: DynamicLibrary.h,v 1.10 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _DYNAMIC_LIBRARY_H
#define _DYNAMIC_LIBRARY_H

#include "common.h"

#ifdef  HAVE_DLFCN_H
#include <dlfcn.h>
#endif

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif

typedef int (*DLPROC)(...);

namespace CaboCha
{
  class DynamicLibrary
  {
  private:
#if defined _WIN32 && ! defined (__CYGWIN__)
    HINSTANCE libHandle;
#else
    void *libHandle;
#endif

  public:
    DynamicLibrary  (): libHandle(0) {};
    DynamicLibrary  (const char *libName) { load (libName); };
    ~DynamicLibrary () { unload (); };

    int load (const char *libName)
    {
      if (libName) {
#if defined _WIN32 && ! defined (__CYGWIN__)
	libHandle = LoadLibrary (libName);
#else
	libHandle = dlopen (libName, RTLD_LAZY);
#endif
      }

      return (libHandle != 0);
    }

    int unload ()
    {
      if (libHandle) {
#if defined _WIN32 && ! defined (__CYGWIN__)
	FreeLibrary (libHandle);
#else
	dlclose (libHandle);
#endif
      }

      libHandle = 0;
      return 1;
    }

    int isloaded() const { return (libHandle != 0); };

    DLPROC getProcAddr (const char *procName) const
    {
      DLPROC proc = 0;

      if (libHandle) {
#if defined _WIN32 && ! defined (__CYGWIN__)
	proc = (DLPROC) GetProcAddress (libHandle, procName);
#else
	proc = (DLPROC) dlsym (libHandle, procName);
#endif
      }

      return proc;
    }
  };
}

#endif
