/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: chasenio.cpp,v 1.19 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "chasenio.h"
#include "param.h"
#include <strstream>
#include <string>

#ifndef _WIN32
extern "C" {
#include <stdio.h>
}
#endif

#if defined _WIN32 && ! defined (__CYGWIN__)
#define CHASEN_DLNAME_WINOLD          "chasen.dll"
#define CHASEN_DLNAME                  "libchasen.dll"
#define CHASEN_INIT_PROC_NAME_WINOLD   "set_argument_parameters"
#define CHASEN_PARSE_PROC_NAME_WINOLD  "analyze"
#else
#define CHASEN_DLNAME                  "libchasen.so"
#endif
#define CHASEN_INIT_PROC_NAME   "chasen_getopt_argv"
#define CHASEN_PARSE_PROC_NAME  "chasen_sparse_tostr"

namespace CaboCha
{
  ChaSenSingleton *ChaSen::instance = 0;

  bool ChaSen::open (Param &p)
  {
    std::string chasenrc = p.getProfileString ("chasenrc");

    if (instance) {
      if (instance->chasenrc != chasenrc ) {
	_what = "ChaSen:ChaSen(): chasen is singleton. you cannot give different chasenrc";
	return false;
      } else {
	return true;
      }
    }

    instance = new ChaSenSingleton;
    instance->chasenrc = chasenrc;

#ifndef CHASEN_USE_STATIC
    if (instance->dl.load (CHASEN_DLNAME)) {
      instance->fpInit  = instance->dl.getProcAddr (CHASEN_INIT_PROC_NAME);
      instance->fpParse = instance->dl.getProcAddr (CHASEN_PARSE_PROC_NAME);
    }

    // invoke old interface
#if defined _WIN32 && ! defined (__CYGWIN__)
    bool winold = false;
    if (! instance->dl.isloaded() && instance->dl.load (CHASEN_DLNAME_WINOLD)) {
      instance->fpInit  = instance->dl.getProcAddr (CHASEN_INIT_PROC_NAME_WINOLD);
      instance->fpParse = instance->dl.getProcAddr (CHASEN_PARSE_PROC_NAME_WINOLD);
      winold = true;
    }
#endif

    if ( ! instance->dl.isloaded() || ! instance->fpInit || ! instance->fpParse) {
      _what = "ChaSen::open(): Cannot load dynamic link library: ";
      _what += CHASEN_DLNAME;
      instance->dl.unload ();
      return false;
    }
#endif

    int size = 0;
    char *option[6];
    option[0] = "CaboCha::ChaSen";

    if (chasenrc.empty()) {
      size = 1;
    } else {
      option[1] = "-r";
      option[2] = const_cast<char *>(chasenrc.c_str());
      size = 3;
    }
    option[size] = 0;

#ifndef CHASEN_USE_STATIC
#if defined _WIN32 && ! defined (__CYGWIN__)
    if (winold) (*(instance->fpInit))(size,   option);
    else (*(instance->fpInit))(option, stderr);
#else
    (*(instance->fpInit))(option, stderr);
#endif
#else
    chasen_getopt_argv (option, stderr);
#endif
    return true;
  }

  Tree* ChaSen::parse (Tree *tree)
  {
#ifndef CHASEN_USE_STATIC
    char *p = (char *)(*(instance->fpParse))(tree->sentence.c_str());
#else
    char *p = (char *)chasen_sparse_tostr(const_cast<char *>(tree->sentence.c_str()));
#endif
    std::istrstream istr (p, strlen (p));
    tree->read (istr, INPUT_POS);
    tree->outputLayer = OUTPUT_POS;
    return tree;
  };
}
