/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: chunker.cpp,v 1.14 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "chunker.h"
#include "param.h"
#include <iterator>

namespace CaboCha
{
  bool Chunker::open (Param &p)
  {
    partial_parsing = p.getProfileInt ("candidate");
    action_mode     = p.getProfileInt ("action-mode");
    ne              = p.getProfileInt ("ne");
    std::string filename = p.getProfileString ("chunker-model");

    if (action_mode == PARSER_MODE) {
      if (filename.empty()) {
	_what = "Chunker::open (): model file is empty";
	return false;
      }

      char *tmp[] = {0, "-m", 0, "-C", 0};
      tmp[0] = const_cast<char *>(p.getSystemName());
      tmp[2] = const_cast<char *>(filename.c_str());

      chunker = yamcha_new (4, tmp);
      if (! chunker) {
	_what = std::string("Chunker::open ():") + yamcha_strerror (chunker);
	return false;
      }
    } else {
      partial_parsing = 1;
    }

    return true;
  }

  Tree *Chunker::parse (Tree *tree)
  {
    if (!tree) return 0;

    size_t size = tree->size ();

    // build features
    yamcha_clear (chunker);

    // process each token
    for (size_t i = 0; i < size; i++) {
      feature.clear();

      const char *surface = (*tree)[i].surface;
      const char *pos     = (*tree)[i].pos;
      const char *ctype   = (*tree)[i].ctype;
      const char *cform   = (*tree)[i].cform;
      const char *_ne     = (*tree)[i].ne;

      feature.push_back (surface);
      feature.push_back (pos);
      feature.push_back (ctype[0] ? ctype : "*");
      feature.push_back (cform[0] ? cform : "*");

      int status = partial_parsing ? (*tree)[i].status : TOKEN_ANY;
      if (ne == NE_WITH_CONSTRAINT && _ne[0] == 'I') status = TOKEN_NEVER;

      if (action_mode == PARSER_MODE) {

	if (status == TOKEN_BEGIN)      feature.push_back ("B");
	else if (status == TOKEN_NEVER) feature.push_back ("I");
	yamcha_add (chunker, feature.size(), (char **)&feature[0]);

      } else {

	std::copy (feature.begin (), feature.end(), std::ostream_iterator <std::string> (std::cout, " "));

	if (i == 0 || (*tree)[i].chunk) {
	  std::cout << "B\n";
	} else {
	  std::cout << "I\n";
	}

      }
    }

    if (action_mode == PARSER_MODE) {

      yamcha_parse (chunker);

      for (size_t i = 0; i < size; i++) {
	if (i == 0 || strcmp (yamcha_get_tag(chunker, i), "B") == 0)
	  (*tree)[i].chunk = tree->allocChunk ();
      }

    } else {
      std::cout << '\n';
    }

    tree->outputLayer = OUTPUT_CHUNK;

    return tree;
  }
}
