/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: common.h,v 1.28 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CABOCHA_COMMON_H
#define _CABOCHA_COMMON_H

// tricky macro for MSVC
#ifdef _MSC_VER
#define for if(0); else for
#endif

#define COPYRIGHT "CaboCha (Yet Another Japanese Dependency Structure Analyzer)\n\
Copyright (C) 2001-2004 Taku Kudo, All rights reserved.\n"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

#include <set>
#include <stdexcept>
#include <algorithm>

namespace CaboCha
{
  using namespace std;

  template <class T> inline T _min(T x,T y) { return (x<y)?x:y; }
  template <class T> inline T _max(T x,T y) { return (x>y)?x:y; }

  static inline int str2int (const char *str)
  {
    char *nptr;
    int l = strtol (str, &nptr, 10);
    if (str == nptr)
      throw std::runtime_error (std::string (str) + ": invalid integer");
    return l;
  }

  static inline int str2int (const char *str, int min, int max)
  {
    char *nptr;
    int l = strtol (str, &nptr, 10);
    if (str == nptr ||l < min || l > max)
      throw std::runtime_error (std::string (str) + ": invalid integer");
    return l;
  }

  static inline std::string createFileName (const std::string &path, const std::string &file)
  {
    std::string s = path;
#if defined (_WIN32) && !defined (__CYGWIN__)
    if (s.size () && s[s.size()-1] != '\\') s += '\\';
#else
    if (s.size () && s[s.size()-1] != '/') s += '/';
#endif
    s += file;
    return s;
  }

  static inline void removeFileName (std::string &s)
  {
    int len = (int)s.size() - 1;
    bool ok = false;
    for (; len >= 0; --len) {
#if defined (_WIN32) && !defined (__CYGWIN__)
      if (s[len] == '\\') { ok = true; break; }
#else
      if (s[len] == '/')  { ok = true; break; }
#endif
    }
    if (ok) s = s.substr (0, len);
    else s = ".";
  }

  static inline void replaceString (std::string &s, const std::string &src, const std::string &dst)
  {
    size_t pos = s.find (src);
    if (pos != std::string::npos) s.replace (pos, src.size(), dst);
  }

  template <class Iterator>
  static inline size_t tokenize (char *str, char *del, Iterator out, size_t max)
  {
    char *stre = str + strlen (str);
    char *dele = del + strlen (del);
    size_t size = 0;

    while (size < max) {
      char *n = std::find_first_of (str, stre, del, dele);
      *n = '\0';
      *out++ = str;
      ++size;
      if (n == stre) break;
      str = n + 1;
    }

    for (size_t i = size; i < max; ++i) *out++ = "";
    return size;
  }
}

#endif
