/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: dep.h,v 1.14 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CABOCHA_PARSER_H
#define _CABOCHA_PARSER_H
#include "analyzer.h"
#include "cabocha.h"
#include <string.h>
#include <set>
#include <yamcha.h>

namespace CaboCha
{
  class cmpstr {
  public:bool operator () (char *s1, char *s2) { return (strcmp (s1, s2) < 0);}
  };

  struct result_t {
    double score;
    std::string rel;
    int link;
    void reset () { score = 0.0; rel = "O"; link = -1; }
  };

  class DepAnalyzer: public Analyzer
  {
  private:
    yamcha_svm_t *model;
    std::vector < std::vector <char *> > static_feature;
    std::vector < std::vector <char *> > dyn_b_feature;
    std::vector < std::vector <char *> > dyn_b;
    std::vector < std::vector <char *> > dyn_a_feature;
    std::vector < std::vector <char *> > dyn_a;
    std::vector < std::vector <char *> > gap;
    std::vector < std::vector <char *> > gap_list;
    std::vector <result_t> result;
    std::vector <Chunk *>  chunks;

    char   *fp[1024];
    std::set <char *, cmpstr> fpset;
    int  is_write_header;
    int  action_mode;
    int  partial_parsing;
    size_t csize;

    bool         estimate (int, int, std::string&, double&);
    void         resize   (size_t);
    size_t build    (Tree *tree);

  public:
    bool open (Param &);
    Tree *parse (Tree *);

    DepAnalyzer (Param &p): model (0), is_write_header (0) { if (!open(p)) throw std::runtime_error (_what); };
    DepAnalyzer (): model (0), is_write_header (0) {};
    ~DepAnalyzer () { yamcha_svm_destroy (model); };
  };

}
#endif
