/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: freelist.h,v 1.8 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CABOCHA_FREELIST_H
#define CABOCHA_FREELIST_H

#include <cstdlib>
#include <string.h>
#include <vector>
#include <iostream>
#include <stdexcept>

namespace CaboCha {

  template <class T>
  class FreeList
  {
  private:
    std::vector < std::pair <size_t, T *> > freeList;
    size_t pi;
    size_t li;
    size_t default_size;

  public:
    void free () { li = pi = 0; }

    T* alloc (size_t req = 1)
    {
      try {
	while (li < freeList.size ()) {
	  if ( (pi + req + 32) < freeList[li].first ) {
	    T *r = freeList[li].second + pi;
	    pi += req;
	    return r;
	  }
	  li++;
	  pi = 0;
	}
	size_t _size = std::max (req, default_size);
	freeList.push_back (std::make_pair (_size, new T [_size]));
	pi += req;
	return freeList[li].second;
      }

      catch (std::exception &e) {
        std::cerr << "FreeList::alloc (): " << e.what () << std::endl;
        exit (EXIT_FAILURE);
        return 0;
      }
    }

    FreeList (size_t _size): pi(0), li(0), default_size (_size)
    {
    }

    ~FreeList ()
    {
      for (li = 0; li < freeList.size(); li++)
	delete [] freeList[li].second;
    }
  };
}
#endif
