/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: libcabocha.cpp,v 1.12 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "cabocha.h"
#include "param.h"
#include "common.h"
#include <fstream>
#include <iostream>
#include <stdexcept>

static std::string errorStr;

typedef struct _cabocha_t {
  int allocated;
  CaboCha::Parser *ptr;
} cabocha_t;

int cabocha_do (int argc, char **argv)
{
  CaboCha::Parser parser;
  return parser.parse (argc, argv);
}

cabocha_t *cabocha_new (int argc, char **argv)
{
  cabocha_t *c = new cabocha_t;
  CaboCha::Parser *ptr = new CaboCha::Parser;
  if (! c || ! ptr) {
    errorStr = std::string ("cabocha_new(): bad alloc");
    return 0;
  }
  c->allocated = 0;

  if (! ptr->open (argc,argv)) {
    errorStr = std::string ("cabocha_new(): ")  + ptr->what ();
    delete ptr;
    delete c;
    return 0;
  }

  c->ptr = ptr;
  c->allocated = 1;
  return c;
}

cabocha_t *cabocha_new2 (char *arg)
{
  cabocha_t *c = new cabocha_t;
  CaboCha::Parser *ptr = new CaboCha::Parser;
  if (! c || ! ptr) {
    errorStr = std::string ("cabocha_new2(): bad alloc");
    return 0;
  }
  c->allocated = 0;
  if (! ptr->open (arg)) {
    errorStr = std::string ("cabocha_new2(): ")  + ptr->what ();
    delete ptr;
    delete c;
    return 0;
  }

  c->ptr = ptr;
  c->allocated = 1;
  return c;
}

void cabocha_destroy(cabocha_t *c)
{
  if (c && c->allocated) {
    delete c->ptr;
    delete c;
  }
  c = 0;
}

char *cabocha_strerror (cabocha_t *c)
{
  if (! c || ! c->allocated) return const_cast<char *>(errorStr.c_str ());
  return const_cast<char *>(c->ptr->what());
}

#define CABOCHA_CHECK_FIRST_ARG(a,c,t) \
    if (! (c) || ! (c)->allocated) { \
       errorStr = std::string (a) + ": First argment is invalid"; \
           return 0; \
    } \
   CaboCha::Parser *(t) = (c)->ptr;

char *cabocha_sparse_tostr (cabocha_t *c, char *s)
{
  CABOCHA_CHECK_FIRST_ARG("cabocha_sparse_tostr", c, t);
  return const_cast<char*>(t->parseToString (s));
}

char *cabocha_sparse_tostr2 (cabocha_t *c, char *s, size_t len)
{
  CABOCHA_CHECK_FIRST_ARG("cabocha_sparse_tostr2", c, t);
  return const_cast<char*>(t->parseToString (s,len));
}

char *cabocha_sparse_tostr3 (cabocha_t *c, char *s, size_t len, char *s2,  size_t len2)
{
  CABOCHA_CHECK_FIRST_ARG("cabocha_sparse_tostr3", c, t);
  return const_cast<char*>(t->parseToString (s,len,s2,len2));
}

cabocha_tree_t *cabocha_sparse_totree (cabocha_t* c, char *s)
{
  CABOCHA_CHECK_FIRST_ARG("cabocha_sparse_tree", c, t);
  return static_cast<cabocha_tree_t *>(t->parse (s));
}

cabocha_tree_t *cabocha_sparse_totree2 (cabocha_t* c, char *s, size_t len)
{
  CABOCHA_CHECK_FIRST_ARG("cabocha_sparse_tree", c, t);
  return static_cast<cabocha_tree_t *>(t->parse (s, len));
}

size_t cabocha_tree_size (cabocha_tree_t* t)
{
  CaboCha::Tree* t2 = static_cast<CaboCha::Tree *>(t);
  return t2->size ();
}

cabocha_token_t  *cabocha_tree_get_token  (cabocha_tree_t *t, size_t id)
{
  CaboCha::Tree* t2 = static_cast<CaboCha::Tree *>(t);
  return static_cast<cabocha_token_t*>(&(*t2)[id]);
}

char *cabocha_tree_tostr  (cabocha_tree_t* t, int fmt)
{
  CaboCha::Tree* t2 = static_cast<CaboCha::Tree *>(t);
  return const_cast<char *>(t2->toString (fmt));
}

char *cabocha_tree_tostr2  (cabocha_tree_t* t, int fmt, char *out, size_t len)
{
  CaboCha::Tree* t2 = static_cast<CaboCha::Tree *>(t);
  return const_cast<char *>(t2->toString (fmt, out, len));
}

