/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: mecabio.cpp,v 1.3 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mecabio.h"
#include "param.h"
#include <strstream>
#include <string>

#ifndef _WIN32
extern "C" {
#include <stdio.h>
}
#endif

#if defined _WIN32 && ! defined (__CYGWIN__)
#define MECAB_DLNAME "libmecab.dll"
#else
#define MECAB_DLNAME "libmecab.so"
#endif

#define MECAB_INIT_PROC_NAME   "mecab_new"
#define MECAB_CLOSE_PROC_NAME  "mecab_destroy"
#define MECAB_ERROR_PROC_NAME  "mecab_strerror"
#define MECAB_PARSE_PROC_NAME  "mecab_sparse_tostr"


namespace CaboCha
{
  bool MeCab::open (Param &p)
  {
    std::string mecabrc = p.getProfileString ("mecabrc");

#ifndef MECAB_USE_STATIC
    if (dl.load (MECAB_DLNAME)) {
      fpInit  = dl.getProcAddr (MECAB_INIT_PROC_NAME);
      fpClose = dl.getProcAddr (MECAB_CLOSE_PROC_NAME);
      fpError = dl.getProcAddr (MECAB_ERROR_PROC_NAME);
      fpParse = dl.getProcAddr (MECAB_PARSE_PROC_NAME);
    }

    if ( ! dl.isloaded() || ! fpInit || ! fpParse || ! fpError || ! fpClose) {
      _what = "MeCab::open(): Cannot load dynamic link library: ";
      _what += MECAB_DLNAME;
      dl.unload ();
      return false;
    }
#endif

    int size = 0;
    char *option[6];
    option[0] = "CaboCha::MeCab";

    if (mecabrc.empty()) {
      option[1] = "-Ochasen";
      size = 2;
    } else {
      option[1] = "-Ochasen";
      option[2] = "-r";
      option[3] = const_cast<char *>(mecabrc.c_str());
      size = 4;
    }
    option[size] = 0;

#ifndef MECAB_USE_STATIC
    mecab = (mecab_t*) (*fpInit) (size, option);
#else
    mecab = mecab_new (size, option);
#endif

    if (! mecab) {
      _what = "MeCab::open(): ";
#ifndef MECAB_USE_STATIC
      _what += std::string ((char *)(*fpError)(0));
      dl.unload ();
#else
      _what += std::string (mecab_strerror(0));
#endif
      return false;
    }

    return true;
  }

  void MeCab::close ()
  {
#ifndef MECAB_USE_STATIC
    (*fpClose)(mecab);
    dl.unload ();
#else
    mecab_destroy (mecab);
#endif
    mecab = 0;
  }

  Tree* MeCab::parse (Tree *tree)
  {
#ifndef MECAB_USE_STATIC
    char *p = (char *)(*fpParse)(mecab, tree->sentence.c_str());
    if (!p) {
      _what = (char *)(*fpError)(mecab);
      return 0;
    }
#else
    char *p = (char *)mecab_sparse_tostr(mecab, const_cast<char *>(tree->sentence.c_str()));
    if (!p) {
      _what = mecab_strerror (mecab);
      return 0;
    }
#endif
    std::istrstream istr (p, strlen (p));
    tree->read (istr, INPUT_POS);
    tree->outputLayer = OUTPUT_POS;
    return tree;
  };
}
