/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: selector.cpp,v 1.11 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef _WIN32
#include "selector-pat-sjis.h"
#else
#include "selector-pat.h"
#endif

#include "selector.h"
#include <strstream>

namespace CaboCha
{
  bool Selector::open (Param &)
  {
    pat_ipa_func.compile(IPA_FUNC_PAT);
    pat_ipa_head.compile(IPA_HEAD_PAT);
    pat_kutouten.compile(KUTOUTEN_PAT);
    pat_open_bracket.compile(OPEN_BRACKET_PAT);
    pat_close_bracket.compile(CLOSE_BRACKET_PAT);
    pat_dyn_a.compile(DYN_A_PAT);
    pat_case.compile(CASE_PAT);
    return true;
  }

  Tree *Selector::parse (Tree *tree)
  {
    if (!tree) return 0;
    if (! tree->empty() && ! (*tree)[0].chunk) return 0;

    size_t size  = tree->size ();

    for (size_t i = 0; i < size;) {

      size_t fid = i;
      size_t hid = i;

      char *buf = tree->alloc (2048);
      std::ostrstream ostrs (buf, 2048);

      size_t cid = i;

      for (; i < size; ++i) {
	if (i != cid && (*tree)[i].chunk) break;

	if (pat_kutouten == (*tree)[i].surface) {
	  ostrs << " G_PUNC:" << pat_kutouten.which();
	  ostrs << " F_PUNC:" << pat_kutouten.which();
	}

	if (pat_open_bracket == (*tree)[i].surface) {
	  ostrs << " G_OB:" << pat_open_bracket.which();
	  ostrs << " F_OB:" << pat_open_bracket.which();
	}

	if (pat_close_bracket == (*tree)[i].surface) {
	  ostrs << " G_CB:" << pat_close_bracket.which();
	  ostrs << " F_CB:" << pat_close_bracket.which();
	}

	if (pat_ipa_func == (*tree)[i].pos) {
	  fid = i;
	} else if (pat_ipa_head != (*tree)[i].pos) {
	  hid = i;
	}
      }

      if (hid > fid) fid = hid;

      const char *hsurface = (*tree)[hid].surface;
      const char *hpos     = (*tree)[hid].pos;
      const char *hcform   = (*tree)[hid].cform;
      const char *hctype   = (*tree)[hid].ctype;
      const char *fsurface = (*tree)[fid].surface;
      const char *fcform   = (*tree)[fid].cform;
      const char *fctype   = (*tree)[fid].ctype;
      const char *fpos     = (*tree)[fid].pos;

      char **hpos_list = (*tree)[hid].pos_list;
      char **fpos_list = (*tree)[fid].pos_list;

      ostrs << " F_H0:" << hsurface;
      for (char **p = hpos_list; *p; ++p)
	ostrs << " F_H" << (int)(p - hpos_list) + 1 << ":" << *p;

      if (hctype[0]) ostrs << " F_H5:" << hctype;
      if (hcform[0]) ostrs << " F_H6:" << hcform;

      ostrs << " F_F0:" << fsurface;
      for (char **p = fpos_list; *p; ++p)
	ostrs << " F_F" << (int)(p - fpos_list) + 1 << ":" << *p;

      if (fctype[0]) ostrs << " F_F5:" << fctype;
      if (fcform[0]) ostrs << " F_F6:" << fcform;

      if (pat_dyn_a == fpos) ostrs << " A:" << fsurface;
      else if (fcform[0])   ostrs << " A:" << fcform;
      else                  ostrs << " A:" << fpos;

      ostrs << " B:" << hpos;

      if (pat_case == fpos) ostrs << " G_CASE:" << fsurface;

      if (cid == 0)  ostrs << " F_BOS:1";
      if (i == size) ostrs << " F_EOS:1";

      // write to tree
      (*tree)[cid].chunk->head = hid - cid;
      (*tree)[cid].chunk->func = fid - cid;

      ostrs << std::ends;

      size_t s = tokenize (buf + 1, " ", (*tree)[cid].chunk->feature, 60);
      (*tree)[cid].chunk->feature[s] = 0;
    }

    tree->outputLayer = OUTPUT_SELECTION;

    return tree;
  }
};
