/*
  CaboCha -- Yet Another Japanese Dependency Structure Analyzer

  $Id: selector.h,v 1.19 2005/12/24 12:30:35 taku Exp $;

  Copyright (C) 2000-2004 Taku Kudo <taku-ku@is.aist-nara.ac.jp>
  This is free software with ABSOLUTELY NO WARRANTY.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _CABOCHA_SELECTOR_H_
#define _CABOCHA_SELECTOR_H_

#include "analyzer.h"
#include "cabocha.h"
#include "common.h"
#include <string.h>

namespace CaboCha
{
  class PatternMatcher
  {
  private:
    char *opat;
    int pos;
    std::vector <char *> pat;

  public:

    PatternMatcher () : opat(0), pos(0) {};
    ~PatternMatcher () { clear (); }

    void clear ()
    {
      for (size_t i = 0; i < pat.size(); ++i) delete [] pat[i];
      pos = -1;
      pat.clear ();
    }

    bool compile (const char** p)
    {
      clear ();
      for (size_t i = 0; p[i]; ++i) {
	char *s = new char [strlen (p[i]) + 1];
	strcpy (s, p[i]);
	pat.push_back (s);
      }
      return true;
    }

    int search (const char* str)
    {
      pos = -1;
      for (size_t i = 0; i < pat.size(); ++i) {
	if (strncmp ((const char*)pat[i], str, strlen (pat[i])) == 0) {
	  pos = i;
	  return 0;
	}
      }
      return -1;
    }

    const char *which ()
    {
      if (pos == -1 || pos > (int)pat.size()) return 0;
      return pat[pos];
    }

    bool operator==(const char *s) { return (-1 != search(s)); }
    bool operator!=(const char *s) { return (-1 == search(s)); }
  };

  class Selector: public Analyzer
  {
    PatternMatcher pat_kutouten, pat_open_bracket, pat_close_bracket;
    PatternMatcher pat_dyn_a, pat_case;
    PatternMatcher pat_ipa_func, pat_ipa_head;
    PatternMatcher pat_juman_func, pat_juman_head;

  public:
    bool open (Param &);
    Tree *parse (Tree *);

    Selector (Param &p) { if (!open(p)) throw std::runtime_error (_what); };
    Selector () {};
    ~Selector () {};
  };

}
#endif
