%module CaboCha
%include exception.i 
%include arg.i     

%{
#undef do_open
#undef do_close
#include <cabocha.h>
#include <stdexcept>
/* #include <strstream> */

#define _CHUNK_ERROR "Chunk::check(): access violation"
#define _TOKEN_ERROR "Token::check(): access violation"
#define _TREE_ERROR  "Tree::check(): access violation"

namespace SWIG_CaboCha {

  class Chunk {
  private:
    cabocha_chunk_t *chunk;
    char *str;
    char **ptr;
    void check () { if (! chunk || *ptr != str) throw _TOKEN_ERROR; }
  public:
    Chunk (cabocha_chunk_t *_chunk, char *_str, char **_ptr): chunk(_chunk), str(_str), ptr(_ptr) {};
    Chunk (): chunk(0), str(0), ptr(0) {};
    ~Chunk () {};
    int link  ()          { check(); return chunk->link;  }
    char *rel ()          { check(); return chunk->rel;   }
    unsigned int head  () { check(); return chunk->head;  }
    unsigned int func  () { check(); return chunk->func;  }
    double   score ()     { check(); return chunk->score; }
  };

  class Token {
  private:
    cabocha_token_t *token;
    char *str;
    char **ptr;
    void check () { if (! token || *ptr != str) throw _TOKEN_ERROR; }
  public:
    Token (cabocha_token_t *_token, char *_str, char **_ptr): token(_token), str(_str), ptr(_ptr) {};
    Token (): token(0), str(0), ptr(0) {};
    ~Token () {};
    bool hasChunk ()     { check(); return token->chunk != 0; }
    Chunk chunk ()  
    { 
      check();
      if (! token->chunk) throw _TOKEN_ERROR; 
      return Chunk (token->chunk, str, ptr);
    }

    char *surface ()  { check(); return token->surface; }
    char *read ()     { check(); return token->read; }
    char *base ()     { check(); return token->base; }
    char *pos ()      { check(); return token->pos; }
    char *ctype ()    { check(); return token->ctype; }
    char *cform ()    { check(); return token->cform; }
    char *ne ()       { check(); return token->ne; }
    char *optional () { check(); return token->optional; }
  };

  class Tree
  {
  private:
    cabocha_tree_t *tree;
    char *str;
    char **ptr;
    void check () { if (!tree || *ptr != str) throw _TREE_ERROR; }
  public:
    Tree (cabocha_tree_t *_tree, char *_str, char **_ptr): tree(_tree), str(_str), ptr(_ptr) {};
    Tree (): tree(0), str(0), ptr(0) {};
    ~Tree () {};
    unsigned int size    () { check(); return cabocha_tree_size (tree); }
    Token token (unsigned int i) 
    {
      check ();
      if (cabocha_tree_size (tree) <= i) throw _TREE_ERROR;
      return Token (cabocha_tree_get_token (tree, i), str, ptr);
    }
    char *toString (int fmt) { check(); return cabocha_tree_tostr (tree, fmt); }
  };
    
  class Parser
  {
  private:
    cabocha_t *c;
    char *str;
  public:
    Parser (int argc, char *argv[]): c(0), str(0)
    {
      if (! (c = cabocha_new (argc, argv))) throw cabocha_strerror (c);
    }

    ~Parser ()
    {
      delete [] str; str = 0;
      cabocha_destroy (c); c = 0;
    }

    Tree parse (char *p)
    {
      delete [] str;
      str = new char [strlen (p) + 1];
      strcpy (str, p);
      Tree tree (cabocha_sparse_totree (c, str), str, &str);
      return tree;
    }

    char *parseToString (char *p)
    {
      char *r = cabocha_sparse_tostr (c, p);
      if (!r) throw cabocha_strerror (c);
      return r;
    }
  };
}
%}

%exception {
  try { $action }
  catch (std::exception &e) { SWIG_exception (SWIG_RuntimeError, (char*)e.what()); }
  catch (char *e) { SWIG_exception (SWIG_RuntimeError, e); }
  catch (const char *e) { SWIG_exception (SWIG_RuntimeError, (char*)e); }
}

namespace SWIG_CaboCha {
 
  class Chunk {
  public:
    int link  ();         
    char *rel ();            
    unsigned int head  ();
    unsigned int func  ();
    double   score (); 
  };

  class Token {
  public:
    bool hasChunk ();
    Chunk chunk ();  
    char *surface ();
    char *read ();
    char *base ();
    char *pos ();
    char *ctype ();
    char *cform ();
    char *ne ();
    char *optional ();
  };

  class Tree
  {
  public:
    unsigned int size    ();
    Token token (unsigned int);
    char *toString (int);
  };

  class Parser {
  public:
    Parser (int argc, char *argv[]);
    ~Parser ();
    Tree parse (char *);
    char *parseToString (char *);
  };
}

